package com.gyee.alarm.model.vo;

import com.gyee.common.model.PointData;
import com.gyee.common.model.StringUtils;
import lombok.Data;

@Data
public class AlarmTag {


    /**
     * 表名
     */
    private String id;

    /**
     * 时间戳
     */
    private Long ts;

    /**
     * 当前值
     */
    private Double val;

    /**
     * 报警类别，0触发，1触发，-1变化触发，大于等于3ai点触发
     */
    private Integer triggerType=0;

    /**
     * 测点
     */
    private String tagId;

    /**
     * 过去值
     */
    private Double oval=null;

    /**
     * 报警级别
     */
    private Integer rank;
    /**
     * 结束时间戳
     */
    private Long endts;
    /**
     * 报警时长
     */
    private Double timeLong;
    /**
     * 报警描述
     */
    private String description;


    private  Boolean confirmed;

    private String superTalbe;

    /**
     * 故障原因
     */
    private String faultCause;
    /**
     * 类型
     */
    private String characteristic;
    private String alarmId;
    private String deviceId;
    /**
     * 解决方法
     */
    private String resolvent;

    /**
     * 报警类型 /自定义报警 custom，风机 windturbine，场站 station，升压站 booststation
     */
    private String alarmType;

    /**
     * 设备类型，场站 station，风机 windturbine，升压站 booststation
     */
    private String deviceType;

    private Boolean isClose;

    private String wpName;

    private String wpId;
    /**
     * 统一识别码(如果通过uniformCode配置，则设备类型，场站，机型不能为空)
     */
    private String uniformCode;
    /**
     * 更新并检测是否触发报警
     */
    public synchronized Boolean updateAndCheck(PointData pointData) {

        if(null ==oval)
        {
            oval=pointData.getPointValueInDouble();
            return false;
        }
        if (pointData.getPointValueInDouble() == oval) {
            return false;
        }
        ts = pointData.getPointTime();
        oval = pointData.getPointValueInDouble();
        endts= pointData.getPointTime();
        val=0.0;

        if(triggerType==-1)
        {
            val=1.0;
        }else  if(triggerType==0)
        {
            if (oval == 0)
            {
                val=1.0;
            }else
            {
                val=0.0;

            }
        }else  if(triggerType==1)
        {
            if (oval == 1)
            {
                val=1.0;
            }else
            {
                val=0.0;

            }
        }else if(StringUtils.notEmp(uniformCode) && uniformCode.startsWith("BI")){

            int decimal = oval.intValue(); // 十进制报警数
            String binary = Integer.toString(decimal, 2); // 转换为二进制报警字符串

            String reverse = new StringBuffer(binary).reverse().toString();//翻转字符串，便于报警分析
            int value = Integer.valueOf(reverse.charAt(triggerType));
            if (value == 1) {
                val=1.0;
            }else {
                val=0.0;
            }

        }else {
            if (oval.intValue() == triggerType) {
                val=1.0;
            }else {
                val=2.0;
            }

        }

        return true;
    }


    public synchronized Boolean updateAndCheckNoInfo(PointData pointData) {


        ts = pointData.getPointTime();
        oval = pointData.getPointValueInDouble();
        endts= pointData.getPointTime();
        val=0.0;

        if(triggerType==-1)
        {
            val=1.0;
        }else  if(triggerType==0)
        {
            if (oval == 0)
            {
                val=1.0;
            }else
            {
                val=0.0;

            }
        }else  if(triggerType==1)
        {
            if (oval == 1)
            {
                val=1.0;
            }else
            {
                val=0.0;

            }
        }else {
            if (oval.intValue() == triggerType) {
                val=1.0;
            }else {
                val=2.0;
            }

        }

        return true;
    }
}
